local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local sys = require "luci.sys"

m = Map("dnp3-master", "", "")

section_global = m:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = true
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
    translate("Enable DNP3 Master"))
option_enabled.rmempty = false
option_enabled.default = "1"


s = m:section(TypedSection, "dnp3_master", translate("DNP3 Slave devices"))
s.add_toggled = true
s.addremove = true
s.anonymous = true
s.sortable = false
s.dont_show_add = true
s.novaluetext = translate("No DNP3 slaves added yet")
s.template = "cbi/tblsection"

function s.extedit(self, section)
    local connection_id = self.map.uci:get("dnp3-master", section, "connection_id")
    connection_id = (type(connection_id) == "string" and connection_id or "")
    return ds.build_url("admin", "services", "dnp3", "dnp3_master", connection_id)
end

function s.create(self, section, ...)
    local connection_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
    local created = TypedSection.create(self, section, ...)
    self.map:set(created, "connection_id", connection_id)

    if created then
        luci.http.redirect(ds.build_url("admin/services/dnp3/dnp3_master", connection_id))
    end
end

function s.remove(self, section)
    --when deleting slave device, also delete all configured registers
    if (section) then
        local connection_id = self.map.uci:get(self.config, section, "connection_id")
        local object_id = self.map.uci:get(self.config, section, "object_id")

        if (connection_id) then
            m.uci:delete_all("dnp3-master", "dnp3_object",
                function(s) return s.connection == connection_id end)
        end
    end

    m.uci:delete("dnp3-master", section)
    m.uci:save("dnp3-master")
    m.uci:commit("dnp3-master")

    luci.http.redirect(ds.build_url("admin/services/dnp3/dnp3_master"))
end

name = s:option(DummyValue, "name", translate("Name"))
function name.cfgvalue(self, s)
    local set_name = self.map:get(s, "conf_name")
    if set_name then
        if #set_name < 24 then
            return set_name
        else
            return string.sub(set_name, 1, 24) .. "..."
        end
    else
        return "Nameless device"
    end
end

serial_port = s:option(DummyValue, "serial_port", translate("Serial Port"))
function serial_port.cfgvalue(self, s)
    return self.map:get(s, "serial_port") or "N/A"
end

host = s:option(DummyValue, "host", translate("Host"))
function host.cfgvalue(self, s)
    local set_ip = self.map:get(s, "host")
    if set_ip then
        if #set_ip < 24 then
            return set_ip
        else
            return string.sub(set_ip, 1, 24) .. "..."
        end
    else
        return "N/A"
    end
end

port = s:option(DummyValue, "port", translate("Port"))
function port.cfgvalue(self, s)
    return self.map:get(s, "port") or "N/A"
end

enabled = s:option(Flag, "enabled")
enabled.last = true

--Registers--
s3 = m:section(TypedSection, "dnp3_object", "DNP3 Objects")
s3.anonymous = true
s3.template = "cbi/tblsection"

address = s3:option(DummyValue, "dnp3_index", translate("DNP3 Object Index"), translate(""))
function address.cfgvalue(self, s)
    return self.map:get(s, "dnp3_index") or "N/A"
end

name = s3:option(DummyValue, "name", translate("DNP3 Object Name"), translate(""))
function name.cfgvalue(self, s)
    return self.map:get(s, "name") or "N/A"
end

data_type = s3:option(DummyValue, "object_type", translate("Object Type"))
function data_type.cfgvalue(self, s)
    return self.map:get(s, "object_type") or "N/A"
end

local dnp3_to_iec104 = nixio.fs.access("/usr/bin/dnp3-to-iec104")
if dnp3_to_iec104 then
    ioa_type = s3:option(DummyValue, "ioa_type", "IOA Type")
    function ioa_type.cfgvalue(self, s)
        return self.map:get(s, "ioa_type") or "N/A"
    end

    ioa_address = s3:option(DummyValue, "ioa_address", "IOA Address")
    function ioa_address.cfgvalue(self, s)
        return self.map:get(s, "ioa_address") or "N/A"
    end
end

return m
