local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local wa = require "luci.tools.webadmin"
local sys = require "luci.sys"

local connection_id
if arg[1] then
    connection_id = arg[1]
else
    luci.http.redirect(ds.build_url("admin/services/dnp3/dnp3_master"))
end

local dnp3_object_section = "dnp3_object"
local alert_list = "modbus_register"

m = Map("dnp3-master")
m.redirect = ds.build_url("admin/services/dnp3/dnp3_master")

local cfgName
m.uci:foreach("dnp3-master", "dnp3_master", function(s)
    if s.connection_id == connection_id then
        cfgName = s[".name"]
    end
end)

s = m:section(NamedSection, cfgName, "Slave device configuration", "Slave device configuration")
s.add_toggled = true

enabled = s:option(Flag, "enabled", translate("Enabled"), translate("Check to enable this DNP3 slave configuration"))


conf_name = s:option(Value, "conf_name", translate("Device Name"))
conf_name.datatype = "rangelength(1, 32)"

dnp3_type = s:option(ListValue, "type", translate("Type"))
dnp3_type:value("rtu", translate("DNP3 RTU"))
dnp3_type:value("tcp", translate("DNP3 TCP"))
dnp3_type.default = "none"

host = s:option(Value, "host", translate("IP address"))
host.datatype = "ip4addr"
host.default = "127.0.0.1"
host:depends("type", "tcp")

port = s:option(Value, "port", translate("Port"))
port.datatype = "port"
port.default = "20000"
port:depends("type", "tcp")

serial_port = s:option(Value, "serial_port", translate("Serial Port"))
serial_port.datatype = "string"
serial_port.default = "/dev/ttyS0"
serial_port:depends("type", "rtu")

baudrate = s:option(ListValue, "baud", translate("Baud rate"))
baudrate:value("300", "300")
baudrate:value("1200", "1200")
baudrate:value("2400", "2400")
baudrate:value("4800", "4800")
baudrate:value("9600", "9600")
baudrate:value("19200", "19200")
baudrate:value("38400", "38400")
baudrate:value("57600", "57600")
baudrate:value("115200", "115200")
baudrate.default = "115200"
baudrate:depends("type", "rtu")

parity = s:option(ListValue, "parity", translate("Parity"))
parity:value("N", translate("None"))
parity:value("E", translate("Even"))
parity:value("O", translate("Odd"))
parity.default = "N"
parity:depends("type", "rtu")

databits = s:option(ListValue, "data_bit", translate("Data bits"))
databits:value("5", "5")
databits:value("6", "6")
databits:value("7", "7")
databits:value("8", "8")
databits.default = "8"
databits:depends("type", "rtu")

stopbits = s:option(ListValue, "stop_bit", translate("Stop bits"))
stopbits:value("1", "1")
stopbits:value("2", "2")
stopbits.default = "1"
stopbits:depends("type", "rtu")

rtu_dnp_address = s:option(Value, "LocalAddr", "Master Address", translate("DNP3 address of the local device"))
rtu_dnp_address.datatype = "range(1,1024)"
rtu_dnp_address.default = 1;

dnp_master_address = s:option(Value, "RemoteAddr", "DNP Outstation Address",translate("DNP3 address of the remote device"))
dnp_master_address.datatype = "range(1,1024)"
dnp_master_address.default = 2

disableUnsolOnStartup = s:option(ListValue, "disableUnsolOnStartup", "Disable Unsolicited On Startup",translate("If Yes, the master will disable unsol on startup for all 3 classes"))
disableUnsolOnStartup:value("0", "No")
disableUnsolOnStartup:value("1", "Yes")
disableUnsolOnStartup.default = "1"

timeout = s:option(Value, "Timeout", "Response Timeout (In Msec)",translate("The response timeout in milliseconds for confirmed requests"))
timeout.datatype = "integer"
timeout.default = "1000"


app_layer_timeout = s:option(Value, "app_layer_timeout", "App Layer Timeout (In Sec)",translate("Application layer response timeout in seconds"))
app_layer_timeout.datatype = "integer"
app_layer_timeout.default = "5"


integraty_poll_interval = s:option(Value, "integraty_poll_interval","Integraty Poll Interval (In Sec)", translate("Do an integrity poll (Class 3/2/1/0)"))
integraty_poll_interval.datatype = "integer"
integraty_poll_interval.default = "60"


class0_poll_interval = s:option(Value, "class0_poll_interval", "CLASS 0 Poll Interval (In Sec)",translate("Do a Class 0 exception poll in seconds"))
class0_poll_interval.datatype = "integer"
class0_poll_interval.default = "0"

class1_poll_interval = s:option(Value, "class1_poll_interval", "CLASS 1 Poll Interval (In Sec)",translate("Do a Class 1 exception poll in seconds"))
class1_poll_interval.datatype = "integer"
class1_poll_interval.default = "0"

class2_poll_interval = s:option(Value, "class2_poll_interval", "CLASS 2 Poll Interval (In Sec)",translate("Do a Class 2 exception poll in seconds"))
class2_poll_interval.datatype = "integer"
class2_poll_interval.default = "0"

class3_poll_interval = s:option(Value, "class3_poll_interval", "CLASS 3 Poll Interval (In Sec)",translate("Do a Class 3 exception poll in seconds"))
class3_poll_interval.datatype = "integer"
class3_poll_interval.default = "0"


s = m:section(TypedSection, dnp3_object_section, translate("DNP3 Objects"))
s.template = "dnp3_master/tblsection"
s.addremove = true
s.anonymous = true
s.add_toggled = true
s.novaluetext = translate("No requests configured for this slave device yet")

function s:filter(section)
    local connection = self.map.uci:get(self.config, section, "connection")
    if (connection == connection_id) then
        return section
    else
        return nil
    end
end

function s.create(self, section, ...)
    local object_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
    local created = TypedSection.create(self, section, ...)
    self.map:set(created, "object_id", object_id)
    self.map:set(created, "connection", connection_id)

    if created then
        luci.http.redirect(ds.build_url("admin/services/dnp3/dnp3_master", connection_id))
    end
end


--function s.remove(self, section)
    --when deleting slave device, also delete all configured registers
--    local connection = self.map.uci:get("dnp3-master", section, "connection")
--    connection = (type(connection) == "string" and connection or "")

--    if (section) then
--        local query_id = self.map.uci:get(self.config, section, "query_id")

--        if (query_id) then
--            m.uci:delete_all("modbus-master", "modbus_register",
--                function(s) return s.query_id == query_id end)
--        end
--        m.uci:delete("modbus-master", section)
--    end
--    return ds.build_url("admin", "services", "modbus", "modbus_master", connection)
--end


name = s:option(Value, "name", translate("Object Name"))
name.optional = false

object_type = s:option(ListValue, "object_type", translate("DNP3 Object Type"))
object_type:value("BinaryInput", translate("BinaryInput"))
object_type:value("BinaryOutputStatus", translate("BinaryOutputStatus"))
object_type:value("AnalogInput", translate("AnalogInput"))
object_type:value("AnalogOutputStatus", translate("AnalogOutputStatus"))
object_type.default = "BinaryInput"

dnp3_index = s:option(Value, "dnp3_index", translate("DNP3 Object Index"))
dnp3_index.datatype = "range(0,65536)"
dnp3_index.default = "1"
dnp3_index.rmempty = false

local dnp3_to_iec104 = nixio.fs.access("/usr/bin/dnp3-to-iec104")
if dnp3_to_iec104 then
    ioa_type = s:option(ListValue, "ioa_type", "IOA Type")
    ioa_type:value("30", translate(" Single Point Information with timestamp"))
    ioa_type:value("31", translate(" Double Point Information with timestamp"))
    ioa_type:value("34", translate(" Measured Value- normalized value with timestamp"))
    ioa_type:value("35", translate(" Measured Value- scaled value with timestamp"))
    ioa_type:value("36", translate(" Measured Value- short floating point value timestamp"))

    ioa_address = s:option(Value, "ioa_address", "IOA Address")
end


return m
